using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Diagnostics;


namespace ResizeC
{
    #region win32struct
    [StructLayout(LayoutKind.Sequential)]
    public struct POINT
    {
        public int X;
        public int Y;

        public POINT(int x, int y)
        {
            this.X = x;
            this.Y = y;
        }

        public static implicit operator System.Drawing.Point(POINT p)
        {
            return new System.Drawing.Point(p.X, p.Y);
        }

        public static implicit operator POINT(System.Drawing.Point p)
        {
            return new POINT(p.X, p.Y);
        }
    }
    [Serializable, StructLayout(LayoutKind.Sequential)]
    public struct RECT
    {
        public int Left;
        public int Top;
        public int Right;
        public int Bottom;

        public RECT(int left_, int top_, int right_, int bottom_)
        {
            Left = left_;
            Top = top_;
            Right = right_;
            Bottom = bottom_;
        }

        public int Height { get { return Bottom - Top; } }
        public int Width { get { return Right - Left; } }
        public Size Size { get { return new Size(Width, Height); } }

        public Point Location { get { return new Point(Left, Top); } }

        // Handy method for converting to a System.Drawing.Rectangle
        public Rectangle ToRectangle()
        { return Rectangle.FromLTRB(Left, Top, Right, Bottom); }

        public static RECT FromRectangle(Rectangle rectangle)
        {
            return new RECT(rectangle.Left, rectangle.Top, rectangle.Right, rectangle.Bottom);
        }

        public override int GetHashCode()
        {
            return Left ^ ((Top << 13) | (Top >> 0x13))
              ^ ((Width << 0x1a) | (Width >> 6))
              ^ ((Height << 7) | (Height >> 0x19));
        }

        #region Operator overloads

        public static implicit operator Rectangle(RECT rect)
        {
            return rect.ToRectangle();
        }

        public static implicit operator RECT(Rectangle rect)
        {
            return FromRectangle(rect);
        }

        #endregion
    }
    #endregion

    public partial class Form1 : Form
    {
        #region win32api
        [DllImport("user32.dll")]
        static extern IntPtr WindowFromPoint(POINT Point);
        [DllImport("user32.dll")]
        static extern bool GetClientRect(IntPtr hWnd, out RECT lpRect);
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool GetWindowRect(IntPtr hWnd, out RECT lpRect);
        [DllImport("user32.dll")]
        private static extern UInt32 SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int x, int y, int width, int height, int flags);
        [DllImport("user32.dll")]
        private static extern IntPtr GetAncestor(IntPtr hwnd, uint gaFlags);
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        static extern int GetClassName(IntPtr hWnd, StringBuilder lpClassName, int nMaxCount);
        [DllImport("user32.dll")]
        extern static int GetWindowText(IntPtr hWnd, StringBuilder lpStr, int nMaxCount);
        [DllImport("user32.dll")]
        static extern bool IsWindow(IntPtr hWnd);
        [DllImport("user32.dll", EntryPoint = "GetWindowDC", SetLastError = true)]
        static extern IntPtr GetWindowDC(IntPtr hWnd);
        [DllImport("user32.dll")]
        static extern int ReleaseDC(IntPtr hWnd, IntPtr hDC);
        [DllImport("gdi32.dll")]
        static extern bool PatBlt(IntPtr hdc, int nXLeft, int nYLeft, int nWidth, int nHeight, uint dwRop);

        private const int DSTINVERT = 0x00550009;
        private const int GA_ROOT = 2;
        private const int SWP_NOMOVE = 0x0002;
        private const int SWP_NOZORDER = 0x0004;
        private const int SWP_FRAMECHANGED = 0x0020;
        #endregion

        private IntPtr hwndTarget = IntPtr.Zero;
        private string presetPath = Path.GetDirectoryName(Application.ExecutablePath) + @"\Preset.txt";

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// EBhȄɔ]FŘg`
        /// EBhEɑ΂2ĂяoƘg
        /// ǂ̃TCĝ܂pNR[h
        /// </summary>
        private void HilightWindow(IntPtr hwnd)
        {
            if (hwnd == IntPtr.Zero || !IsWindow(hwnd)) { return; }

            IntPtr hdc = GetWindowDC(hwnd);

            RECT rect;
            GetWindowRect(hwnd, out rect);

            rect.Right -= rect.Left;
            rect.Bottom -= rect.Top;
            rect.Top = rect.Left = 0;

            if (!rect.ToRectangle().IsEmpty)
            {
                const int DINV = 3;
                PatBlt(hdc, 0, 0, DINV, rect.Height, DSTINVERT);
                PatBlt(hdc, DINV, 0, rect.Width - DINV, DINV, DSTINVERT);
                PatBlt(hdc, rect.Width - DINV, DINV, DINV, rect.Height - DINV, DSTINVERT);
                PatBlt(hdc, DINV, rect.Height - DINV, rect.Width - 2 * DINV, DINV, DSTINVERT);
            }

            ReleaseDC(hwnd, hdc);
        }

        /// <summary>
        /// EBhẼNCAg̈̃TCYύX
        /// j[2iɂȂꍇɑΉ
        /// </summary>
        private void SetClientSize(IntPtr hwnd, int width, int height)
        {
            RECT rectClient;
            RECT rectClient2;
            RECT rectWindow;

            GetClientRect(hwndTarget, out rectClient);
            GetWindowRect(hwndTarget, out rectWindow);

            int w = rectWindow.Width - rectClient.Width + width;
            int h = rectWindow.Height - rectClient.Height + height;

            SetWindowPos(hwndTarget, IntPtr.Zero, 0, 0, w, h, SWP_NOMOVE | SWP_NOZORDER);

            // TCYύXɃj[2iɂȂꍇ̂߁ATCYύX
            GetClientRect(hwndTarget, out rectClient2);

            if (rectClient2.Height != height)
            {
                h += height - rectClient2.Height;
                SetWindowPos(hwndTarget, IntPtr.Zero, 0, 0, w, h, SWP_NOMOVE | SWP_NOZORDER);
            }
        }

        private void LoadPreset()
        {
            try
            {
                StreamReader reader = new StreamReader(presetPath);
                while (!reader.EndOfStream)
                {
                    string line = reader.ReadLine();
                    comboBoxPreset.Items.Add(line);
                }

                reader.Close();
            }
            catch
            {
            }
        }

        private void SavePreset()
        {
            try
            {
                StreamWriter writer = new StreamWriter(presetPath, false);

                for (int i = 0; i < comboBoxPreset.Items.Count; i++)
                {
                    writer.WriteLine(comboBoxPreset.Items[i].ToString());
                }

                writer.Close();
            }
            catch
            {
            }
        }

        //private void ClearState()
        //{
        //    labelCapture.Text = "ŃEBhEɃhbvĂ";
        //    hwndTarget = IntPtr.Zero;
        //    textBoxHeight.Text = "0";
        //    textBoxWidth.Text = "0";
        //}

        private void labelCapture_MouseDown(object sender, MouseEventArgs e)
        {
            hwndTarget = IntPtr.Zero;

            labelCapture.Capture = true;  // hbȌꍇtrue/false؂ւۂH

            Cursor.Current = Cursors.Hand;
        }

        private void labelCapture_MouseUp(object sender, MouseEventArgs e)
        {
            labelCapture.Capture = false;

            HilightWindow(hwndTarget);  // nCCg
        }

        private void labelCapture_MouseMove(object sender, MouseEventArgs e)
        {
            if (labelCapture.Capture == false) { return; }

            // }EẌʒuɂEBhE擾
            IntPtr hwnd = WindowFromPoint(new POINT(Cursor.Position.X, Cursor.Position.Y));
            if (hwnd != IntPtr.Zero)
            {
                IntPtr hwndParent = GetAncestor(hwnd, GA_ROOT); // qEBhE͂Ȃ

                if (hwndParent != IntPtr.Zero)
                {
                    if (hwndParent != hwndTarget)
                    {
                        // }EẌʒuɂEBhEnCCg
                        HilightWindow(hwndParent);
                        HilightWindow(hwndTarget);
                    }

                    StringBuilder sb = new StringBuilder(1024);
                    GetWindowText(hwndParent, sb, sb.Capacity);
                    labelCapture.Text = sb.ToString();
                    GetClassName(hwndParent, sb, sb.Capacity);
                    labelCapture.Text += "\n" + sb.ToString();

                    // ݂̃NCAg̈̃TCY\
                    RECT rect;
                    GetClientRect(hwndParent, out rect);

                    textBoxWidth.Text = rect.Width.ToString();
                    textBoxHeight.Text = rect.Height.ToString();

                    hwndTarget = hwndParent;
                }
            }
        }

        private void buttonResize_Click(object sender, EventArgs e)
        {
            try
            {
                SetClientSize(hwndTarget, int.Parse(textBoxWidth.Text), int.Parse(textBoxHeight.Text));
            }
            catch
            {
                textBoxWidth.Text = "0";
                textBoxHeight.Text = "0";
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            comboBoxPreset.Items.Add(textBoxWidth.Text + "," + textBoxHeight.Text);
        }

        private void comboBoxPreset_SelectedIndexChanged(object sender, EventArgs e)
        {
            string[] nums = comboBoxPreset.SelectedItem.ToString().Split(',');
            textBoxWidth.Text = nums[0];
            textBoxHeight.Text = nums[1];
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            LoadPreset();

            FileVersionInfo ver = FileVersionInfo.GetVersionInfo(
                System.Reflection.Assembly.GetExecutingAssembly().Location);

            this.Text = string.Format("ResizeC ver{0}.{1}", ver.FileMajorPart, ver.FileMinorPart);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            SavePreset();
        }

        private void menuQuit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

    }
}